# andy@engagemedia.org 
# 13/dec/06
# GPL 
# 
# adapted for use in ATMediaFile , from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/440555
#

############
# BitTorrent server launch
############
import logging,sys,pdb,time,traceback,os
from thread import get_ident,allocate_lock
from operator import mod

#need this prior to BT imports
import gettext
gettext.install('bittorrent', 'locale')
    
from BitTorrent.launchmanycore import LaunchMany
from BitTorrent.defaultargs import get_defaults
from BitTorrent.parseargs import parseargs, printHelp
from BitTorrent import configfile
from BitTorrent import BTFailure
from BitTorrent.bencode import bdecode
from BitTorrent.makemetafile import make_meta_files

from twisted.spread import pb
from twisted.internet import reactor
from twisted.internet import threads

import urllib

class DataLock:
    def __init__(self):
    	print "DataLock __init__"
        self.threadlock = allocate_lock()
    def lock(self): 
    	self.threadlock.acquire()
    def unlock(self): 
    	self.threadlock.release()

def downloadDir():
    ddir=None
    if sys.platform=='win32':
        ddir = os.environ.get('HOMEPATH')
        if ddir and len(ddir)>0:
            ddir = '%s%sMy Documents%sTorrentServer' % (ddir,os.sep,os.sep)
            if not os.path.isdir(ddir): os.makedirs(ddir)
        else: ddir='.'
    else:
        ddir = os.environ.get('HOME')
        try:
            ddir += '/Documents/TorrentServer'
            if not os.path.isdir(ddir): os.makedirs(ddir)
        except: ddir = os.environ.get('HOME')
    return ddir
    
class TorrentServer(pb.Root):
    
    torrentPort = 9119
    log = None
    tServer = None
    isProcess = True

    def __init__(self, td):
        TorrentServer.tServer = self
        self.launcher = None
        self.torrentDir = td
        self.torrentLock = DataLock()
        self.status = ''
        self.torrents = {}
        self.stoppedTorrents = {}
        self.moduloCount = 0

    def listen(self):
        TorrentServer.log.debug('init(%s): listening '%get_ident())
        reactor.listenTCP( TorrentServer.torrentPort, pb.PBServerFactory(self))
        reactor.run()


    def remote_DownloadMakeTorrentAndSeed(self,bsInfo):
    	#parse args
        bootstrapURL=bsInfo['bootstrapURL']
	trackerURL=bsInfo['trackerURL']
	filename=bsInfo['filename']
	incoming_comment=bsInfo['comment']

	# XXX make all these operations run in a separate thread.
	# this then requires a separate mechanism to insert the bittorrent file contents back into plone , in an asynchronous manner

    	print "downloading media file from url %s, making torrent file with tracker info %s, filename %s, comment %s " % (bootstrapURL,trackerURL,filename,incoming_comment)
	#download it 
	incoming_fp = downloadDir() + '/' + filename
	urllib.urlretrieve(bootstrapURL, incoming_fp)

	#make_meta_file
	print "retrieved file .. making meta_file ..."
	torrent_filepath = incoming_fp + ".torrent"
	make_meta_files( trackerURL, [incoming_fp], piece_len_pow2=18, comment=incoming_comment, target = torrent_filepath)
	print "done making %s " % torrent_filepath

	# this server regularly polls the directory for new torrents, and will seed it automatically

	#return the .torrent file contents, back to the calling function
	# (within plone, atmediafile tool)
	return open(torrent_filepath,"r").read()

    def remote_UpdateExecutionStatus(self, execStatus):
        self.torrentLock.lock()
        try:
            TorrentServer.log.debug('remote_UpdateExecutionStatus(%s): %s' %\
                                   (get_ident(),execStatus))
            try:
                td = self.torrentDir
                ttdd = execStatus.get('torrent_dir',td)
                self.status = execStatus.get('status','')
                self.torrents = execStatus.get('torrents',{})
                if td and ttdd!=td:
                    self.torrentDir = ttdd
                    self.status = 'restart'
                elif not td: self.torrentDir = ttdd
            except:
                traceback.print_exc()
        finally: self.torrentLock.unlock()

    def remote_ReportProgressStatus(self):
        TorrentServer.log.debug('remote_ReportProgressStatus: ')
        self.torrentLock.lock()
        try: return {'torrents':self.torrents, 'status': self.status}
        finally: self.torrentLock.unlock()

    def initTorrent(self):
        self.torrentLock.lock()
        self.status=''
        uiname = 'btlaunchmany'
        defaults = get_defaults(uiname)
        try:
            config, args = configfile.parse_configuration_and_args(defaults, uiname, [], 0, 1)
            #config, args = configfile.parse_configuration_and_args(defaults, uiname, sys.argv[1:], 0, 1)
            config['torrent_dir'] = self.torrentDir
            config['parse_dir_interval'] = 20 #make the dir scan happen @20 seconds, not default of 60
            self.config = config
        except BTFailure, e:
            traceback.print_exc()
            TorrentServer.log.error(_("%s\nrun with no args for parameter explanations")) % str(e)
        self.torrentLock.unlock()
        if self.torrentDir: self.runTorrents()

    def runTorrents(self):
        TorrentServer.log.debug('runTorrents(%s): LaunchMany... %s'%\
                               (get_ident(), self.torrentDir))
        self.launcher = LaunchMany(self.config, self, 'btlaunchmany')
	print "self.launcher is %s " % (self.launcher.__class__)
        self.launcher.run()
        TorrentServer.log.debug('runTorrents(%s): DONE with torrents...'%get_ident())
        if self.status=='quit':
            if TorrentServer.isProcess:
                reactor.stop()
                #sys.exit()
        else:
            if self.status=='restart':
                log.debug('torrentServer(): Will restart %s '%self.torrentDir)
                self.initTorrent()
            if self.torrentDir: self.runTorrents()
            
    def display(self, data):
        #self.torrentLock.lock()
        try:
            if self.status == 'quit': return True
            if self.status=='restart': return True
            while self.status=='paused':
                TorrentServer.log.debug( 'display(%s): is paused' % (get_ident()))
                #self.torrentLock.unlock()
                time.sleep(1.0)
                #self.torrentLock.lock()

            self.moduloCount += 1
            modulo = mod(self.moduloCount, 3)

            for xx in data:
                ( name, status, progress, peers, seeds, seedsmsg, dist,
                  uprate, dnrate, upamt, dnamt, size, t, msg ) = xx
                if status is not 'downloading':
                    #TorrentServer.log.debug( 'display(%s): %s: %s (%s)' % (get_ident(),status, name, progress))
		    pass

                stopstatus = self.torrents.get(name)
                if stopstatus and (stopstatus[0]=='cancel' or stopstatus[0]=='stop'):
                    try: os.remove(name)
                    except: traceback.print_exc()
                    del self.torrents[name]
                else:
                    self.torrents[name] = ['progress',progress]
            del data
            return False
        finally: 
	    #self.torrentLock.unlock()
	    #print "finished display!"
	    pass

    def message(self, str):
        TorrentServer.log.debug('FeedbackReporter.message(): %s'%str)

    def exception(self, str):
        TorrentServer.log.warn('FeedbackReporter: exception=%s'%str)

    def didEndTorrentThread(self,foo=None):
        TorrentServer.log.debug('didEndTorrentThread(): %s'%foo)

    def didEndTorrentThreadErr(self,foo=None):
        TorrentServer.log.debug('didEndTorrentThreadErr(): %s'%foo)



def main(args):
    if __debug__:
        level = logging.DEBUG
    else:
        level = logging.WARN

    logging.basicConfig(level=level, format='%(asctime)s %(levelname)s %(message)s')
    handler = logging.StreamHandler()
    TorrentServer.log = logging.getLogger('TorrentServer')
    TorrentServer.log.addHandler(handler)
    TorrentServer.log.propagate = 0

    TorrentServer.log.debug('torrentServer.main(): will load config')
    TorrentServer.tServer = TorrentServer(downloadDir())
    dfr = threads.deferToThread(TorrentServer.tServer.initTorrent)
    dfr.addCallback(TorrentServer.tServer.didEndTorrentThread)
    dfr.addErrback(TorrentServer.tServer.didEndTorrentThreadErr)
    
    TorrentServer.tServer.listen()

if __name__=="__main__":
    main(sys.argv)



