# File: testATMediaFile_rename.py
# 
# Copyright (c) 2005 by Chris Miles
# Generator: manual - this couldn't be generated by AGX as it needs to
#     use ATMediaExample to perform the ATMediaFile tests.
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#
__author__  = '''Chris Miles <miles.chris@gmail.com>'''
__docformat__ = 'plaintext'

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import ATMFTestCase
from OFS.Image import File

#
# Base TestCase for ATMediaFile
#
from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase
from Products.ExternalStorage import config

ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PortalTransforms', quiet=1)
ZopeTestCase.installProduct('MimetypesRegistry', quiet=1)
ZopeTestCase.installProduct('ATMediaFile')
ZopeTestCase.installProduct('ATMediaExample')	## Added manually...
# If the products's config includes DEPENDENCIES, install them too
try:
    from Products.ATMediaFile.config import DEPENDENCIES
except:
    DEPENDENCIES = []
for dependency in DEPENDENCIES:
    ZopeTestCase.installProduct(dependency)

#PRODUCTS = ('Archetypes', 'ATMediaFile')
PRODUCTS = ('Archetypes', 'ATMediaExample')

PloneTestCase.setupPloneSite(products=PRODUCTS)


class TestRenameFile(ATMFTestCase.ATMFTestCase):
    """ TestCase for ATMediaFile: object rename functionality.
    """

    def afterSetUp(self):
        self.art = self._addFileByFileUpload(self.folder, 'file',
                                             ATMFTestCase.FILENAME_TEXT)
	self.basepath = os.path.join( ATMFTestCase.BASE_PATH,
		PloneTestCase.default_user )
	self.path = os.path.join( self.basepath,
		ATMFTestCase.FILENAME_TEXT )
        self.path_new = os.path.join(self.basepath, ATMFTestCase.FILENAME_TEXT_NEW)
        self.title = self.art.Title()
        if config.IS_PLONE_2_0:
            origin = ATMFTestCase.FILENAME_TEXT
        else:
            origin = '/'.join(self.art.getPhysicalPath())
        self.folder.folder_rename([origin],
                                  [ATMFTestCase.FILENAME_TEXT_NEW], ['new title'])


    def testOldRemovedFromZODB(self):
        self.failIf(ATMFTestCase.FILENAME_TEXT in self.folder.objectIds())

    def testOldRemovedFromFileSystem(self):
        self.failIf(ATMFTestCase.FILENAME_TEXT in os.listdir(self.basepath))

    def testNewPresentOnZODB(self):
        self.failUnless(ATMFTestCase.FILENAME_TEXT_NEW in self.folder.objectIds())

    def testNewPresentOnFileSystem(self):
        self.failUnless(ATMFTestCase.FILENAME_TEXT_NEW in
                        os.listdir(self.basepath))

    def testPathChangedOnStorage(self):
        art_new = getattr(self.folder, ATMFTestCase.FILENAME_TEXT_NEW)
        field = art_new.getField('file')
        self.assertEqual(self.path_new,
                         field.storage.getFilepath(art_new, 'file'))

    def testSameContentOnZODB(self):
	fp = open( ATMFTestCase.DATAPATH_TEXT, 'r' )
	content = fp.read()
	fp.close()
        art_new = getattr(self.folder, ATMFTestCase.FILENAME_TEXT_NEW)
        file_obj = art_new.getFile()
        self.assertEqual(content, str(file_obj.data))

    def testTypeContentOnZODB(self):
        art_new = getattr(self.folder, ATMFTestCase.FILENAME_TEXT_NEW)
        file_obj = art_new.getFile()
        self.failUnless(isinstance(file_obj, File))

    def testSameContentOnFileSystem(self):
	fp = open( ATMFTestCase.DATAPATH_TEXT, 'r' )
	content = fp.read()
	fp.close()
        self.assertEqual(content, open(self.path_new, 'r').read())

    def testTitleChanged(self):
        art_new = getattr(self.folder, ATMFTestCase.FILENAME_TEXT_NEW)
        self.assertNotEqual(self.title, self.art.Title())
        self.assertEqual('new title', self.art.Title())

    def testFilenameChanged(self):
        art_new = getattr(self.folder, ATMFTestCase.FILENAME_TEXT_NEW)
        field = art_new.getField('file')
        self.assertEqual(ATMFTestCase.FILENAME_TEXT_NEW,
                         field.storage.getFilename(art_new, 'file'))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestRenameFile))
    return suite



if __name__ == '__main__':
    framework()


