"""This package adds extensions to portal_catalog.
"""
from Acquisition import aq_inner
from Products.CMFPlone.CatalogTool import registerIndexableAttribute
from Products.ATVideo.interfaces.content_markers import IEngageVideo
from Products.CMFCore.utils import getToolByName
from zope.interface import providedBy
from DateTime import DateTime

from Products.ATVideo.config import TRANSCODING_STATUSES

def isTranscodedATEngageVideoObj(object, portal, **kw):
    if not IEngageVideo.providedBy(object):
        return None
    statuses = TRANSCODING_STATUSES
    status = str(object.getIndyTubeStatus())
    if status in statuses:
	video_status = statuses.get(status, (False, u""))
	if video_status[0]:
		indytube_html = object.getIndyTubeHTML()
	else:
		indytube_html = ""
    else:
	video_status = (False, u"")
	indytube_html = ""

    return { 'transcoding_status':video_status[0], 'indytube_html':indytube_html }

def isPublishableATEngageVideoObj(object, portal, **kw):
    if not IEngageVideo.providedBy(object):
        return None
   
    portal_workflow = getToolByName(portal,'portal_workflow') 
    portal_membership = getToolByName(portal,'portal_membership')
    portal_contentlicensing = getToolByName(portal,'portal_contentlicensing')

    #wf state
    item_state = portal_workflow.getInfoFor(object, 'review_state', '')
    #name of creator 
    member_name = object.Creator()
    #get the actual user object
    user = portal_membership.getMemberById(member_name)
    portal.plone_log("Item %s by %s is in state %s. user is %s " % (object.absolute_url(), member_name, item_state,user))
    if user is None:
        portal.plone_log("No matching members??")

    if user is not None and item_state == 'published':
        (url,length,type) = object.getFileAttribs()
	cclicense = portal_contentlicensing.getLicenseAndHolderFromObject(object)
	cclicense_text = portal_contentlicensing.DefaultSiteLicense[0]
	cclicense_url  = None
	if cclicense[1][1] != 'None':
		cclicense_text = cclicense[1][1] 
	if cclicense[1][2] != 'None':
		cclicense_url  = cclicense[1][2] 
	

        d = { 'published':True, 
              'item_title': object.Title(),
              'item_creator_email': user.getProperty('email',''),
              'item_creator_fullname':user.getProperty('fullname',''),
              'subject': object.Subject(),
              'item_rfc822_datetime': DateTime(object.Date()).rfc822(),
              'item_rfc3339_datetime': DateTime(object.Date()).HTML4(),
              'file_url': url,
              'file_length':length,
              'file_type':type,
	      'item_url':object.absolute_url(),
	      'license_text':cclicense_text,
	      'license_url':cclicense_url
            }

    else: 
        d = {'published': False }

    portal.plone_log("Returning metadata %s " % d)
    return d

print "registering new index for ATEngageVideo"
registerIndexableAttribute('isPublishableATEngageVideoObj', isPublishableATEngageVideoObj)
registerIndexableAttribute('isTranscodedATEngageVideoObj', isTranscodedATEngageVideoObj)

