# -*- coding: utf-8 -*-

# Imports: Zope
from Acquisition import aq_inner
from Products.Five  import BrowserView
from zope.event import notify
from zope.interface import implements

# Imports: CMF
from Products.CMFCore.utils import getToolByName

# Import Plone
from Products.CMFPlone.utils import normalizeString
from Products.CMFPlone import MessageFactory
mf = MessageFactory('portletification')

# Imports: ATCT
from Products.ATContentTypes.interface.topic import IATTopic

# Imports: Portletification interfaces
from Products.Portletification.interfaces import IPortlet
from Products.Portletification.interfaces import IPortletifiable
from Products.Portletification.interfaces import IPortletRetriever
from Products.Portletification.interfaces import IListingContentProvider

# Imports: Portletification events
from Products.Portletification.events import ObjectTaggedAsPortlet
from Products.Portletification.events import ObjectUntaggedAsPortlet



class ListingContentProvider( BrowserView ):
    """ Access contents of a folder """

    implements( IListingContentProvider )

    def __init__(self, context, request):
        self.context = context
        self.request = request


    def getLimitedContents(self, limit=5):
        if IATTopic.providedBy(self.context):
            return self.context.queryCatalog()[:limit]
        return self.context.getFolderContents()[:limit]


    def getContents(self):
        if IATTopic.providedBy(self.context):
            return self.context.queryCatalog()
        return self.context.getFolderContents()

    
    def normalizeString(self, text, context=None, encoding=None, relaxed=False):
        return normalizeString(text, context=self.context, encoding=encoding, relaxed=relaxed)


    def getViewUrl(self, brain):
        props = getToolByName(self.context, 'portal_properties')       
        view_action_types = props.site_properties.getProperty('typesUseViewActionInListings', ())
    
        if brain.portal_type in view_action_types:
            return brain.getURL() + '/view'
    
        return brain.getURL()



class PortletRetriever( BrowserView ):

    implements( IPortletRetriever )

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.catalog = getToolByName(context, 'portal_catalog')


    def getLeftPortlets(self):
        b = self.catalog({ 'portletified': 'left',
                           'sort_on': 'getObjPositionInParent',
                           'object_implements': 'Products.Portletification.interfaces.IPortlet' })
        return b


    def getRightPortlets(self):
        b = self.catalog({ 'portletified': 'right',
                           'sort_on': 'getObjPositionInParent',
                           'object_implements': 'Products.Portletification.interfaces.IPortlet' })
        return b



class PortletActionProvider( BrowserView ):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.portlets = None
        
        adapted = IPortlet.providedBy(context) and \
                  IPortletifiable(context, None) or \
                  None
        if adapted:
            self.portlets = adapted.portletified()


    def tag_left( self ):
        if self.portlets:
            return not self.portlets['left']
        return False


    def tag_right( self ):
        if self.portlets:
            return not self.portlets['right']
        return False


    def untag_left( self ):
        if self.portlets:
            return self.portlets['left']
        return False


    def untag_right( self ):
        if self.portlets:
            return self.portlets['right']
        return False



class PortletProvider( BrowserView ):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.portlet = IPortletifiable(context, None)


    def portletified(self):
        return self.portlet.portletified()

    
    def portletify(self, position):
        """
        This method will simply tag the item.
        The object will, then, be add to the portlets
        at the given position.
        """
        r = self.portlet.portletify(position = position)
        notify(ObjectTaggedAsPortlet(aq_inner(self.context)))
        
        if r:
            msg = mf('portletified_%s' % position,
                     'This object has been tagged portlet and will be displayed on the %s' % position)
        else:
            msg = mf('portletification_failure',
                     'You failed to tag this object as a portlet')

        self.context.plone_utils.addPortalMessage(msg)
        referer = self.request.get("HTTP_REFERER", self.context.absolute_url())
        self.request.RESPONSE.redirect(referer)


    def unportletify(self, position):
        """
        This method will simply untag the item.
        The object will, then, be removed from the portlets
        at the given position.
        """
        r = self.portlet.unportletify(position = position)
        notify(ObjectUntaggedAsPortlet(aq_inner(self.context)))
        if r:
            msg = mf('unportletified_%s' % position,
                     'This object has been untagged portlet and will be no longer displayed on the %s' % position)
        else:
            msg = mf('unportletification_failure',
                     'You failed to untag this object as a portlet')

        self.context.plone_utils.addPortalMessage(msg)
        referer = self.request.get("HTTP_REFERER", self.context.absolute_url())
        self.request.RESPONSE.redirect(referer)
