# -*- coding: utf-8 -*-

from zope.schema import Int, List, Dict, Choice, Bool, Tuple
from zope.app.event.interfaces import IObjectModifiedEvent
from zope.interface import Interface
from zope import i18n

_ = i18n.MessageIDFactory("vaporisation")


class ITreeUpdateEvent( IObjectModifiedEvent ):
    """
    This triggers the rebuilding of the whole tree
    """


class ISteamer( Interface ):
    """
    The steamer releases the pression by letting the steam out.
    Here, the vaporisation is done !
    """
    def update_tags_tree(self):
        """
        This is the restricted access for a manager to handle his cloud.
        It will simply trigger the private method of the vaporized cloud.
        Once refreshed, it will display a message, acknowledging the change.
        """

    def getVaporizedCloud(self):
        """
        This method return as list of dictionnaries.
        A dict contains :
        - name : the name of the tag. (unicode)
        - weight : the weight of the tag. (int)
        - size : the size of the font, for the HTML rendering. (int)
        """

    def isJointNavigation(self):
        """
        This method is a simple public accessor for the attribute 'joint' that
        defines a cloud with the joint navigation activated. It returns a bool.
        """

    def currentSubjects(self):
        """
        This method return the list of keywords actually selected.
        The result is an iterable sequence of unicodes.
        """

    def removableTags(self):        
        """
        This method return the list of keywords actually selected.
        The result is an iterable sequence of dictionaries :
        - name : the name of the tag. (unicode)
        - link : an http link that will be used in the template, for the tag.
        """

    def getLinkPath(self):
        """
        This method calculates the link that will be used in the cloud HTML
        generation. It takes in consideration the currently selected keywords.
        This is an internal method. Never to be used outside the view itself.
        """


class IPortletifiable( Interface ):
    """
    marker interface
    """


class IVaporizedCloud( Interface ):
    """
    A cloudy bunch of keywords
    """
    # storing the tags
    keywords = List(title=u"The list of keywords")
    all_keys = List(title=u"The list of all the keywords")
    tagsTree = Dict(title=u"The full tree of tags")

    # informations about weights
    lowest  = Int(title=u"lowest weight")
    highest = Int(title=u"heighest weight")


class ICustomizableCloud( Interface ):
    """
    Customizable parts of the cloud
    """
    # Customization    
    steps = Int(
        title=_(u"Number of different sizes"),
        description=_(u"This number will also determine the biggest size."),
        required=True,
        default=10
        )

    limit = Int(
        title=_(u"Number of tags to display"),
        description=_(u"Zero displays it all."),
        required=False,
        default=0
        )
    
    joint = Bool(
        default=True,
        required=True,
        title=_(u"Activate joint navigation"),
        description=_(u"Joint navigation puts keywords together"
                      u"for associative searches.")
        )

    restrict = Tuple(
        required=False,
        title=_(u"Remove from keywords list"),
        description=_(u"Restrict the cloud keywords by"
                      u"removing these keywords"),
        value_type=Choice(vocabulary='vaporisation.keywords'),
        )
